use crate::services::service::Service;
use crate::store::object::Obj;

pub enum Props {
    CreationDate,
    GetLastModified,
    GetETag,
    GetContentType,
    RessourceType,
    GetContentLength,
    GetContentLanguage,
    DisplayName,
    FileId,
    Permissions,
    Size,
    HasPreview,
    Favorite,
    CommentsUnread,
    OwnerDisplayName,
    ShareTypes,
    ContainedFolderCount,
    ContainedFileCountm,
}

impl From<&Props> for &str {
    fn from(variant: &Props) -> Self {
        match variant {
            Props::CreationDate => "<d:creationdate />",
            Props::GetLastModified => "<d:getlastmodified />",
            _ => todo!("Props conversion not implemented"),
        }
    }
}

pub struct ReqProps {
    service: Service,
    properties: Vec<Props>,
}

impl ReqProps {
    pub fn new(path: &str) -> Self {
        ReqProps {
            service: Service::new(),
            properties: Vec::new(),
        }
    }

    pub fn get_properties(mut self, properties: Vec<Props>) -> Self {
        self.properties.extend(properties);
        self
    }

    pub fn get_property(&mut self, property: Props) {
        self.properties.push(property);
    }

    fn get_body(&self) -> String {
        let mut xml = String::from(
            r#"<?xml version="1.0" encoding="UTF-8"?><d:propfind xmlns:d="DAV:" xmlns:oc="http://owncloud.org/ns" xmlns:nc="http://nextcloud.org/ns"><d:prop>"#,
        );
        for property in &self.properties {
            xml.push_str(property.into());
        }
        xml.push_str(r#"</d:prop></d:propfind>"#);
        xml
    }

    pub fn send(&mut self) {
        self.service
            .propfind(String::from(
                "http://localhost:8080/remote.php/dav/files/admin/Documents",
            ))
            .send();
    }
}

impl From<Obj> for ReqProps {
    fn from(obj: Obj) -> Self {
        ReqProps::new("")
    }
}
