use core::ops::{Deref, DerefMut};
use reqwest::blocking::{Client, ClientBuilder, RequestBuilder};
use reqwest::{header::HeaderMap, Method,  Url};

const USER_AGENT: &str = "Nextsync";

pub struct ClientConfig {
    client: Option<ClientBuilder>,
}

impl ClientConfig {
    pub fn new() -> Self {
        ClientConfig {
            client: Some(Client::builder().user_agent(USER_AGENT)),
        }
    }

    pub fn default_headers(mut self, headers: HeaderMap) -> Self {
        self.client = Some(self.client.take().expect("Client was already built").default_headers(headers));
        self
    }

    pub fn build(&mut self) -> Client {
        self.client.take().expect("Cannot build the client twice").build().unwrap()
    }
}

pub struct Service {
    client: ClientConfig,
    method: Option<Method>,
    url: Option<String>,
    headers: HeaderMap,
    body: Option<String>,
}

impl Service {
    pub fn new() -> Self {
        Service {
            client: ClientConfig::new(),
            method: None,
            url: None,
            headers: HeaderMap::new(),
            body: None,
        }
    }

    pub fn propfind(&mut self, url: String) -> &mut Self {
        self.method = Some(Method::from_bytes(b"PROPFIND").expect("Cannot be invalid"));
        self.url = Some(url);
        self
    }

    pub fn send(&mut self) {
        dbg!(self.client
            .build()
            .request(
                self.method.clone().expect("Method must be set"),
                self.url.clone().expect("Url must be set"),
            )
            .bearer_auth("rK5ud2NmrR8p586Th7v272HRgUcZcEKIEluOGjzQQRj7gWMMAISFTiJcFnnmnNiu2VVlENks")
            .send());
    }
}
